import box
import click
import pathlib
import questionary
import semver


def locate_pyproject_toml(path) -> pathlib.Path:
    path = pathlib.Path(path)
    if path.is_dir():
        return path / "pyproject.toml"
    return path


def bump(
    path: str | pathlib.Path,
    bump_type: str,
    yes: bool = False,
    quiet: bool = False,
    dry_run: bool = False,
):
    """
    Bumps the version of a project specified in a pyproject.toml file.
    Args:
        path (str | pathlib.Path): The path to the project directory or pyproject.toml file.
        bump_type (str): The type of version bump ('major', 'minor', 'patch').
        yes (bool, optional): If True, automatically confirm the version bump. Defaults to False.
        quiet (bool, optional): If True, suppress output messages. Defaults to False.
        dry_run (bool, optional): If True, do not actually write changes to the file. Defaults to False.
    Raises:
        AttributeError: If the bump_type is not a valid method of semver.Version.
        FileNotFoundError: If the pyproject.toml file is not found in the specified path.
    Returns:
        None
    """
    path = pathlib.Path(path)

    file_path = locate_pyproject_toml(path)

    data, version, semver_version = read_pyproject_data(file_path)

    new_version = get_bumped_version(bump_type, semver_version)

    if not quiet:
        click.secho(
            f"Bumping {bump_type} version from {version} to {new_version}", fg="green"
        )

    if dry_run:
        click.secho(f"Dry run mode. New version: {str(new_version)}", fg="yellow")
        return

    answer = True

    if not yes:
        answer = questionary.confirm("Do you want to continue?").ask()
        print(answer)

    if answer:
        data.project.version = str(new_version)
        with open(file_path, "w") as f:
            f.write(data.to_toml())
        if not quiet:
            click.secho("Version bumped successfully", fg="green")

def get_bumped_version(bump_type, semver_version) -> semver.Version:
    """
    Bumps the given semantic version based on the specified bump type.
    Args:
        bump_type (str): The type of version bump to perform. 
                         It should be one of 'major', 'minor', or 'patch'.
        semver_version (object): An instance of a semantic version class that 
                                 supports bumping methods like bump_major, bump_minor, and bump_patch.
    Returns:
        object: A new instance of the semantic version class with the version bumped accordingly.
    """

    fn = f"bump_{bump_type}"
    callable = getattr(semver_version, fn)
    new_version = callable()
    return new_version

def read_pyproject_data(file_path) -> tuple[box.Box, str, semver.Version]:
    """
    Reads the pyproject.toml file and extracts version information.

    Args:
        file_path (str): The path to the pyproject.toml file.

    Returns:
        tuple: A tuple containing:
            - data (box.Box): The parsed data from the pyproject.toml file.
            - version (str): The version string from the pyproject.toml file.
            - semver_version (semver.Version): The parsed semantic version information.
    """
    with open(file_path) as f:
        data = box.Box.from_toml(f.read())
    version = data.project.version

    semver_version = semver.Version.parse(version)
    return data,version,semver_version


def version(path):
    """
    Retrieves and prints the current version of a project from its pyproject.toml file.

    Args:
        path (str): The file path to the directory containing the pyproject.toml file.

    Raises:
        FileNotFoundError: If the pyproject.toml file is not found at the specified path.
        box.BoxError: If there is an error parsing the TOML data.

    Example:
        >>> version("/path/to/project")
        ProjectName current version: 1.0.0
    """
    path = locate_pyproject_toml(path)
    with open(path) as f:
        data = box.Box.from_toml(f.read())
    click.secho(
        f"{data.project.name} [{path}] current version: {data.project.version}", fg="green"
    )
